/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
 *  Author: Andreas Brauchli <andreas.brauchli@aizo.com>
 */

/**
 * A function that converts stored scene responder triggers from the old to the new version
 */
function convertOldTriggers() {
    LOG.logln('Converting triggers from old scene responder app');
    Property.load();
    var ndTriggers = Property.getNode('triggers');
    if (!ndTriggers) {
        return;
    }
    var triggers = ndTriggers.getChildren();
    var i;
    for (i = 0; i < triggers.length; ++i) {
        var ndOldParams = triggers[i].getChild('params'),
            oldParams,
            oldTrigger,
            oldActions;

        if (!ndOldParams) {
            LOG.logln('Error: no old params found to convert on trigger ' + triggers[i].name);
            deleteOldTrigger(triggers[i]);
            continue;
        }

        try {
            // LOG.logln('Debug: old params: ' + ndOldParams.getValue());
            oldParams = JSON.parse(ndOldParams.getValue());
            // LOG.logln('Debug: old trigger: ' + decodeURIComponent(oldParams.trigger));
            oldTrigger = JSON.parse(oldParams.trigger);
            // LOG.logln('Debug: old actions: ' + decodeURIComponent(oldParams.actions));
            oldActions = JSON.parse(oldParams.actions);
        } catch (e) {
            LOG.logln('Deleting old trigger '+ triggers[i].name +': failed to parse');
            deleteOldTrigger(triggers[i]);
            continue;
        }

        //
        // validity checks
        //

        if (!oldTrigger
            || isNaN(oldTrigger.scene)
            || isNaN(oldTrigger.zone)
                // allowed to be unset in very old versions (< 0.1.4)
            || (oldTrigger.group && isNaN(oldTrigger.group))
        ) {
            // delete invalid triggers
            LOG.logln('Deleting old trigger '+ triggers[i].name +': missing trigger');
            deleteOldTrigger(triggers[i]);
            continue;
        }

        if (!validScene(oldTrigger.scene)) {
            LOG.logln('Deleting old trigger '+ triggers[i].name +': scene ' + oldTrigger.scene +' is not supported by this version');
            deleteOldTrigger(triggers[i]);
            continue;
        }

        if (!oldActions
            || !oldActions.actions
            || !(oldActions.actions.length > 0)
        ) {
            // delete invalid triggers
            LOG.logln('Deleting old trigger '+ triggers[i].name +': missing actions');
            deleteOldTrigger(triggers[i]);
            continue;
        }
        oldActions = oldActions.actions;

        //
        // Conversions
        //

        // new params object
        var params = {
            name: triggers[i].name, // or oldParams.name
            disabled: false, // there existed no possibility to disable an action
            scene: parseInt(oldTrigger.scene, 10),
            zone: parseInt(oldTrigger.zone, 10)
        };

        // Convert delay
        if (oldParams.duration === undefined) {
            params.delay = 0;
        } else if (isNaN(oldParams.duration)) {
            LOG.logln('Warning: invalid delay on trigger '+ triggers[i].name +', assuming 0');
            params.delay = 0;
        } else {
            params.delay = 60 * parseInt(oldParams.duration, 10);
        }

        // Convert trigger group
        if (isNaN(oldTrigger.group)) {
            LOG.logln('Warning: no trigger group is set on '+ triggers[i].name +': defaulting to group 1 (Lights)');
            params.group = 1;
        } else {
            params.group = parseInt(oldTrigger.group, 10);
        }

        // Convert old actions
        params.actions = [];
        var a;
        for (a = 0; a < oldActions.length; ++a) {
            var p = {};
            if (isNaN(oldActions[a].zone)
                || (isNaN(oldActions[a].scene) && isNaN(oldActions[a].value))
                || (oldActions[a].group && isNaN(oldActions[a].group))
                || (!isNaN(oldActions[a].value)
                    && (oldActions[a].value < 0 || oldActions[a].value > 255))
            ) {
                LOG.logln('Error: no or invalid action set on trigger '+ triggers[i].name +' action at index '+ a +'. Skipping');
                continue;
            }
            p.zone = parseInt(oldActions[a].zone, 10);
            if (oldActions[a].group === undefined) {
                LOG.logln('Warning: no group is set on '+ triggers[i].name +' action at index '+ a +', defaulting to group 1 (Lights)');
                p.group = 1;
            } else {
                p.group = parseInt(oldActions[a].group, 10);
            }
            if (oldActions[a].scene !== undefined) {
                if (!validScene(oldActions[a].scene)) {
                    LOG.logln('Warning unsupported scene '
                            + oldActions[a].scene +' call from action '+ a
                            +' on '+ triggers[i].name +': skipping');
                    continue;
                }

                p.scene = parseInt(oldActions[a].scene, 10);
            } else if (oldActions[a].value !== undefined) {
                p.outputValue = parseInt(oldActions[a].value, 10);
            }
            params.actions.push(p);
        }
        if (params.actions.length === 0) {
            LOG.logln('Error: no valid actions on trigger '+ triggers[i].name +': deleting');
            deleteOldTrigger(triggers[i]);
            continue;
        }

        // store converted trigger
        deleteOldTrigger(triggers[i]);
        addTrigger(params);
        LOG.logln('Converted trigger '+ params.idname);
    }
    Property.store();
} // convertOldTriggers

/**
 * Deletes an old (and converted but not saved) trigger from the property tree
 */
function deleteOldTrigger(ndTrigger) {
    ndTrigger.getParent().removeChild(ndTrigger);
    Property.store();
}
