/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
 *  Author: Andreas Brauchli <andreas.brauchli@aizo.com>
 */

Ext.namespace('DSS.addon.sceneresponder');

/**
 * An action panel to display scene responses
 * @class DSS.addon.sceneresponder.Panel
 * @extends Ext.grid.Panel
 */
Ext.define('DSS.addon.sceneresponder.Panel', {
    extend: 'Ext.grid.Panel',
    requires: ['DSS.component.ActionModel'],
    uses: [
        'DSS.addon.sceneresponder.Editor',
        'DSS.addon.sceneresponder.Model',
        'DSS.component.ActionType',
        'DSS.component.TargetType',
        'DSS.json.Event',
        'Ext.button.Button',
        'Ext.data.Model',
        'Ext.data.Store',
        'Ext.grid.Column',
        'Ext.selection.RowModel',
        'Ext.window.Window'
    ],

    config: {
        enableColumnHide: false,
        enableColumnMove: false,
        selType: 'rowmodel',
        viewConfig: { forceFit: true }
    },

    /** {Object} The event store */
    store: null,
    /** {Object} The window holding the sceneResponderEditor */
    editorWindow: null,
    /** {Object} The currently selected automaton as DSS.addon.sceneresponder.Model (or null if none) */
    curSelectedAction: null,

    constructor: function(config) {
        this.initConfig(config);
        this.callParent(arguments);
    },

    initComponent: function() {
        var me = this;
        me.store = Ext.create('Ext.data.Store', { model: 'DSS.addon.sceneresponder.Model' });

        me.sceneResponderEditor = Ext.create('DSS.addon.sceneresponder.Editor', me.config);

        me.editorWindow = Ext.create('Ext.window.Window', {
            closeAction: 'hide',
            title: me.appName,
            modal: true, // Exclusive input for the window
            height: 500,
            width: 750,
            layout: 'fit',
            items: me.sceneResponderEditor
        });

        me.sceneResponderEditor.on({
            save: function(values) {
                var srm = Ext.create('DSS.addon.sceneresponder.Model', {
                    name: values.name,
                    delay: values.delay,
                    trigger: values.trigger,
                    actions: values.actions
                });
                me.editorWindow.disable();

                if (me.curSelectedAction) {
                    me.curSelectedAction.update(srm);
                } else {
                    srm.publish(function() {
                        // Note that the action is not certain to have been
                        // processed on the dSS.
                        // It might still be in the event queue.
                        me.store.loadData([srm], true);
                    });
                    // reset the current selection
                    var sel = me.getSelectionModel().getSelection();
                    if (sel.length > 0) {
                        me.curSelectedAction = sel[0];
                    }
                }
                me.editorWindow.hide();
                me.editorWindow.enable();
                me.sceneResponderEditor.resetComponent();
            },
            cancel: function() {
                me.editorWindow.hide();
                // reset the currently selected line (was remove to add a new entry)
                if (!me.curSelectedAction) {
                    var sel = me.getSelectionModel().getSelection();
                    if (sel.length > 0) {
                        me.curSelectedAction = sel[0];
                    }
                }
                me.sceneResponderEditor.resetComponent();
            }
        }); // sceneResponderEditor.on()
        me.on({
            selectionchange: function(view, selections, options) {
                var me = this;
                if (selections.length > 0) {
                    me.curSelectedAction = selections[0];
                    Ext.getCmp('btn-edit').setDisabled(false);
                    Ext.getCmp('btn-delete').setDisabled(false);
                    var btnDisable = Ext.getCmp('btn-disable');
                    btnDisable.setDisabled(false);
                    me.setDisabledButton(me.curSelectedAction.get('disabled'));
                } else {
                    me.curSelectedAction = null;
                    Ext.getCmp('btn-edit').setDisabled(true);
                    Ext.getCmp('btn-disable').setDisabled(true);
                    Ext.getCmp('btn-delete').setDisabled(true);
                }
            }
        });
        me.callParent(arguments);
        me.initPage();
    }, // initComponent

    initPage: function() {
        var me = this,
            aptStore = null,
            onAptStoreLoaded = function() {
                // Disable custom scene name loading for now
                // DSS.apartment.SceneStore.preloadCustomNameCache(function() {
                    me.store.load();
                // });
            };

        // Utterly ugly hack to fully initialize the component
        me.editorWindow.show();
        me.editorWindow.hide();
        // end of utterly ugly hack

        Ext.getCmp('btn-add').handler = function() {
            me.curSelectedAction = null;
            me.editorWindow.show();
        };
        Ext.getCmp('btn-edit').handler = function() {
            me.sceneResponderEditor.resetComponent(me.curSelectedAction);
            me.editorWindow.show();
        };
        Ext.getCmp('btn-disable').handler = function() {
            var st = !me.curSelectedAction.get('disabled');
            me.curSelectedAction.disable(st, function(m) {
                if (me.curSelectedAction === m) {
                    me.setDisabledButton(st);
                }
            });
        };
        Ext.getCmp('btn-delete').handler = function() {
            me.curSelectedAction.kill();
        };
        aptStore = DSS.component.ActionModel.getApartmentStore();
        if (aptStore.isLoading()) {
            me.setLoading(true);
            aptStore.on('load', onAptStoreLoaded);
        } else {
            onAptStoreLoaded();
        }
    },

    viewConfig: {
        stripeRows: true
    },

    tbar: {
        items: [{
            text: _('New automaton'),
            id: 'btn-add',
            iconCls: 'icon-add-action'
            // handler: function() { /* set in initComponent */ }
        }, {
            text: _('Edit automaton'),
            id: 'btn-edit',
            iconCls: 'icon-edit-action',
            disabled: true
            // handler: function() { /* set in initComponent */ }
        }, {
            text: _('Disable automaton'),
            id: 'btn-disable',
            iconCls: 'icon-stop-action',
            disabled: true
            // handler: function() { /* set in initComponent */ }
        }, '->', {
            text: _('Delete automaton'),
            id: 'btn-delete',
            iconCls: 'icon-delete-action',
            disabled: true
            // handler: function() { /* set in initComponent */ }
        }]
    },

    columns: [{
        id: 'col-status',
        header: _('Active'),
        dataIndex: 'disabled',
        renderer: function(disabled) {
            return '<input type="checkbox" disabled="true"'
                    + (!disabled ? ' checked="true">' : '>');
        },
        resizable: false,
        width: 50
    }, {
        id: 'col-name',
        header: _('Name'),
        dataIndex: 'name',
        width: 200
    }, {
        id: 'col-trigger',
        header: _('Trigger'),
        dataIndex: 'id', // alibi index
        maintainFlex: true,
        flex: 2,
        renderer: function(val, metadata, record, colIdx, store, view) {
            return record.trigger().first().get('text');
        },
        sortable: false
    }, {
        id: 'col-actions',
        header: _('Actions'),
        dataIndex: 'id', // alibi index
        maintainFlex: true,
        flex: 5,
        sortable: false,
        renderer: function(val, metadata, record, colIdx, store, view) {
            var actions = record.actions(),
                num = actions.getCount(),
                res = [],
                i;

            for (i = 0; i < num; ++i) {
                res.push(actions.getAt(i).get('text'));
            }
            return res.join('<br>');
        }
    }, {
        id: 'col-delay',
        header: _('Delay'),
        dataIndex: 'delay',
        renderer: function(val) {
            var pad = function(i) { return (i < 10 ? '0'+i : i); };
            var sec = val;
            var min = Math.floor(sec / 60);
            sec -= min * 60;
            var hou = Math.floor(min / 60);
            min -= hou * 60;
            return pad(hou) +':'+ pad(min) +':'+ pad(sec);
        },
        width: 80
    }],
    setDisabledButton: function(val) {
        var btnDisable = Ext.getCmp('btn-disable');
        if (val) {
            btnDisable.setText(_('Enable automaton'));
            btnDisable.setIconCls('icon-start-action');
        } else {
            btnDisable.setText(_('Disable automaton'));
            btnDisable.setIconCls('icon-stop-action');
        }
    }
}); // DSS.addon.sceneresponder.Panel

/**
 * Scene Responder Editor Panel
 * The panel will always have at least one action
 * @class DSS.addon.sceneresponder.Editor
 * @extends Ext.panel.Panel
 */
Ext.define('DSS.addon.sceneresponder.Editor', {
    extend: 'Ext.panel.Panel',
    mixins: {
        observable: 'Ext.util.Observable'
    },
    requires: [
        'DSS.component.ActionType',
        'DSS.component.TargetType'
    ],
    uses: [
        'DSS.component.ActionConfigurator',
        'Ext.data.Model',
        'Ext.data.Store',
        'Ext.form.FieldContainer',
        'Ext.form.field.Text'
    ],
    config: {
        appName: ''
    },
    /**
     * Gets the activity configurator component
     * that holds the apartment store
     * be careful of what you do with it, esp. filtering
     * @return {Object} The activity configurator or null if not initialized
     */
    getActivityConfigurator: function() {
        return this.activityConfigurator;
    },
    /** The action configuration component for the actions */
    actionConfigurator: null,
    /** The action selection component for the trigger scene */
    triggerActionSelector: null,
    /** Textfield holding the action name */
    actionNameBox: null,
    /** Fieldset holding the action delay fields */
    delayBox: null,
    constructor: function(config) {
        this.initConfig(config);
        this.mixins.observable.constructor.call(config);
        this.callParent(arguments);
    },
    initComponent: function() {
        var me = this;

        me.addEvents({
            /**
             * Fired upon cancel button press
             * @event cancel
             */
            cancel: true,
            /**
             * Fired upon save button press
             * @event save
             * @param values {Object} The parameters of valid actions
             */
            save: true
        });

        me.triggerActionSelector = Ext.create('DSS.component.ActionSelector', {
            bindTargetType: DSS.component.TargetType.ZONE,
            bindActionType: DSS.component.ActionType.ACTIVITY
        });
        me.actionConfigurator = Ext.create('DSS.component.ActionConfigurator');
        me.actionNameBox = Ext.create('Ext.form.field.Text', {
            name: 'actionname',
            fieldLabel: _('Name'),
            allowBlank: false,
            width: 350
        });
        me.delayBox = Ext.create('Ext.form.FieldContainer', {
            itemId: 'delay',
            fieldLabel: _('Delay'),
            layout: 'column',
            defaultType: 'numberfield',
            defaults: {
                allowBlank: false,
                allowDecimals: false,
                width: 50
            },
            items: [{
                itemId: 'hours',
                value: 0,
                minValue: 0
            }, {
                itemId: 'minutes',
                value: 0,
                minValue: 0,
                maxValue: 59
            }, {
                itemId: 'seconds',
                value: 0,
                minValue: 0,
                maxValue: 59
            }, {
                xtype: 'label',
                text: '(' + _('hh:mm:ss') + ')',
                align: 'middle',
                margin: '5 0 0 10'
            }]
        });
        Ext.apply(me, {
            layout: 'border',
            items: [
                {
                    region: 'north',
                    bodyPadding: 5,
                    items: [{
                        xtype: 'fieldset',
                        title: _('Properties'),
                        items: [
                            me.actionNameBox,
                            me.delayBox, {
                                xtype: 'fieldset',
                                title: _('Action Trigger'),
                                items: [me.triggerActionSelector]
                            }
                        ]
                    }]
                }, {
                    region: 'center',
                    layout: 'fit',
                    items: [me.actionConfigurator]
                }
            ], // items
            buttons: [
                {
                    text: _('Abort'),
                    handler: function() {
                        me.fireEvent('cancel');
                    }
                }, {
                    text: _('Save'),
                    handler: function() {
                        if (me.isValid()) {
                            me.fireEvent('save', me.getValues());
                        }
                    }
                }
            ]
        }); // Ext.apply

        // Initialize
        this.callParent(arguments);
    }, // initComponent
    /**
     * Reset all values to default (unset) or specified values
     * @param values {Object} (optional) DSS.addon.sceneresponder.Model values to reset to - default if unset
     */
    resetComponent: function(values) {
        var me = this;

        if (values === undefined) {
            me.triggerActionSelector.resetComponent();
            me.actionConfigurator.resetComponent();

            me.actionNameBox.setValue('');
            me.actionNameBox.clearInvalid();
            me.delayBox.getComponent('hours').setValue(0);
            me.delayBox.getComponent('minutes').setValue(0);
            me.delayBox.getComponent('seconds').setValue(0);
            Ext.getCmp('btn-add-action').setDisabled(true);

        } else {
            me.triggerActionSelector.resetComponent(values.trigger().first());
            me.actionConfigurator.resetComponent(values.actions().data.items);

            me.actionNameBox.setValue(values.get('name'));
            var ts = [0, 0, values.get('delay')];
            ts[1] = Math.floor(ts[2] / 60);
            ts[2] -= ts[1] * 60;
            ts[0] = Math.floor(ts[1] / 60);
            ts[1] -= ts[0] * 60;
            me.delayBox.getComponent('hours').setValue(ts[0]);
            me.delayBox.getComponent('minutes').setValue(ts[1]);
            me.delayBox.getComponent('seconds').setValue(ts[2]);
        }

    }, // resetComponent

    isValid: function() {
        return (
            this.actionNameBox.isValid()
            && this.triggerActionSelector.isValid()
            && this.actionConfigurator.isValid()
        );
    },

    /**
     * Returns the currently set form values
     * @return {Object} Object of the form:
     *      name: {String} Name given to the action
     *      delay: {Number} delay in seconds
     *      trigger: {Object} a DSS.component.ActionModel object
     *      actions: {Array} Array of DSS.component.ActionModel objects
     */
    getValues: function() {
        var me = this;
        return {
            name: me.actionNameBox.getValue(),
            delay: (60 * 60 * me.delayBox.getComponent('hours').getValue())
                + (60 * me.delayBox.getComponent('minutes').getValue())
                + me.delayBox.getComponent('seconds').getValue(),
            trigger: me.triggerActionSelector.getValues(),
            actions: me.actionConfigurator.getValues()
        };
    }
});

/**
 * Scene responder storage model
 * @class DSS.addon.sceneresponder.Model
 * @extends Ext.data.Model
 */
Ext.define('DSS.addon.sceneresponder.Model', {
    extend: 'Ext.data.Model',
    fields: [
        'id',
        { name: 'isTmpId', type: 'boolean', defaultValue: false, persist: false },
         'name',
        { name: 'disabled', type: 'boolean', defaultValue: false },
        { name: 'delay', type: 'integer' }
    ],
    hasMany: [
        { name: 'trigger', model: 'DSS.component.ActionModel' },
        { name: 'actions', model: 'DSS.component.ActionModel' }
    ],
    proxy: {
        type: 'ajax',
        actionMethods: { read: 'GET' },
        url: '/json/property/query?query=/scripts/scene-responder'
         + '/triggers/*(id,name,disabled,delay,scene,group,zone)'
         + '/actions/*(blink,scene,outputValue,group,zone,device)',
        pageParam: undefined,
        limitParam: undefined,
        startParam: undefined,
        reader: {
            type: 'json',
            root: 'result.triggers',
            successProperty: 'ok'
        }
    },
    /** @private */
    getTmpId: function() {
            return 'tmp' + Math.floor(Math.random() * 100000000000);
    },
    constructor: function(data, id, raw) {
        var me = this,
            rawTrigger = null,
            rawActions = null,
            trigger = null,
            actions = [];

        if (!raw) {
            if (!id) {
                data.id = me.getTmpId();
                data.isTmpId = true;
            }
            trigger = data.trigger;
            delete data.trigger;
            actions = data.actions;
            delete data.actions;
        } else {
            if (!raw.id) {
                // 0.5.2 and older versions did not have the id field.
                data.id = data.name;
            }
            if (raw.actions) {
                rawActions = raw.actions;
                // delete the array to prevent the Ext parser from parsing
                // these into ActionModels
                delete raw.actions;

                rawTrigger = {
                    group: raw.group,
                    zone: raw.zone,
                    scene: raw.scene
                };
                delete raw.group;
                delete raw.zone;
                delete raw.scene;
            }
            try {
                data.name = decodeURIComponent(raw.name);
            } catch (e) { }
        }

        // Init the model and set default values
        me.callParent(arguments);

        me.actions().clearFilter();
        me.trigger().clearFilter();

        if (raw) {
            // This node was loaded from the server, not newly created
            // create actions and trigger
            Ext.Array.each(rawActions, function(a) {
                actions.push(DSS.component.ActionModel.reconstructFromIds(a));
            });
            trigger = DSS.component.ActionModel.reconstructFromIds(rawTrigger);
        }
        me.actions().loadData(actions);
        me.trigger().loadData([trigger]);
    },
    /**
     * Get the content of this Model as Object that is ready for
     * serialization and compatible to the dSS format
     * @return {Object} This Model's values
     */
    getParams: function() {
        var trg = this.trigger().first(),
            actions = [];

        this.actions().each(function(a) {
            var act = {};

            if (a.get('targetType') === DSS.component.TargetType.ZONE) {
                act.zone = a.zone().first().getId();
                act.group = a.group().first().getId();
            } else if (a.get('targetType') === DSS.component.TargetType.DEVICE) {
                act.device = a.device().first().getId();
            }

            if (a.get('actionType') === DSS.component.ActionType.ACTIVITY) {
                act.scene = a.scene().first().getId();
            } else if (a.get('actionType') === DSS.component.ActionType.OUTPUT) {
                // rescale to 0 - 255
                act.outputValue = Math.ceil(a.get('outputValue') * 2.55);
            } else if (a.get('actionType') === DSS.component.ActionType.BLINK) {
                act.blink = true;
            }
            actions.push(act);
        });

        return {
            id: this.getId(),
            name: this.get('name'),
            delay: this.get('delay'),
            disabled: this.get('disabled'),
            zone: trg.zone().first().getId(),
            group: trg.group().first().getId(),
            scene: trg.scene().first().getId(),
            actions: actions
        };
    },

    /**
     * Publish this action to the dSS
     * @param handler {Function} Callback issued when sent to the dSS
     */
    publish: function(handler) {
        if (this.get('isTmpId')) {
            this.publishTempId(handler);
        } else {
            this.publishDefault(handler);
        }
    },

    /**
     * @private
     * Publish this event to the dSS
     */
    publishDefault: function(handler) {
        var me = this,
            action = Ext.create('DSS.json.Event', {name: 'scene-responder'}),
            params = Ext.clone(me.getParams()),
            hdl = {
                failure: function() {
                    Ext.Msg.alert(
                        _('Error'),
                        _('Couldn\'t create trigger action on server')
                    );
                }
            };
        params.name = encodeURIComponent(params.name);

        if (Ext.isFunction(handler)) {
            hdl.success = Ext.bind(handler, me, [me]);
        }
        action.raise({
                action_type: 'add',
                params: Ext.JSON.encode(params)
            },
            hdl
        );
    },

    /**
     * @private
     * Publish this action with temporary id to the dSS and reassign
     * the definitive id with this model
     */
    publishTempId: function(handler) {
        var me = this,
            // We have a temporary id, get the real one generated
            // by the dSS on creation before passing it on to the UI
            subscr = Ext.create('DSS.json.EventSubscription'),
            evt = 'scene-responder.create.' + me.getId();

        subscr.subscribe(evt, function() {
            me.publishDefault();
            subscr.get(
                function(res) {
                    var i,
                        id;

                    if (res) {
                        for (i = 0; i < res.length; ++i) {
                            if (res[i].name === evt && res[i].properties) {
                                id = res[i].properties.id;
                                break;
                            }
                        }
                    }
                    if (id) {
                        me.setId(id);
                        me.commit();
                        Ext.callback(handler, me);
                    } else {
                        Ext.Msg.alert(
                            _('Action possibly not created'),
                            _('Please reload this page before continuing')
                        );
                    }
                    subscr.unsubscribeAll();
                },
                10000 // 10s timeout
            );
        });
    },

    /**
     * Update this action on the dSS using a new model
     * Note that the id will be copied over to the new model
     * @param newmodel The new model with the edited values
     */
    update: function(newmodel) {
        var me = this,
            st = me.store,
            id = me.getId(),
            index = me.store.indexOf(me);

        if (isNaN(parseInt(id, 10))) {
            // convert the old id from pre 0.5.2 versions
            // for this we need to delete the old action manually
            // from the dSS but we only do so after publishing the new
            newmodel.commit();
            newmodel.publish(function() {
                me.kill(function() {
                    st.insert(index, [newmodel]);
                });
            });

        } else {
            me.store.remove(me);
            newmodel.setId(id);
            newmodel.set('isTmpId', false);
            newmodel.commit();
            newmodel.publish(function() {
                st.insert(index, [newmodel]);
            });
        }
    },
    /**
     * Delete this event on the dSS
     */
    kill: function(callback) {
        var me = this,
            event = Ext.create('DSS.json.Event', {name: 'scene-responder'});

        event.raise(
            {
                action_type: 'delete',
                trigger_id: me.getId()
            },
            function() {
                me.store.remove(me);
                if (Ext.isFunction(callback)) {
                    Ext.callback(callback);
                }
            }
        );
    },
    /**
     * Enable or disable this action on the dSS
     * @param val {Boolean} True to enable, false to disable
     * @param callback {Function} Callback with this model as argument. Issued when sent to the dSS
     */
    disable: function(val, callback) {
        var me = this;
        if (me.get('disabled') == val) {
            if (Ext.isFunction(callback)) {
                Ext.callback(callback, me, [me]);
            }
            return;
        }
        var event = Ext.create('DSS.json.Event', {name: 'scene-responder'});
        event.raise({
                action_type: (val ? 'disable' : 'enable'),
                trigger_id: me.getId()
            },
            function() {
                me.set('disabled', val);
                me.commit();
                if (Ext.isFunction(callback)) {
                    Ext.callback(callback, me, [me]);
                }
            }
        );
    }
});
